import { __spreadArray, __read } from 'tslib';
import { Helpers } from 'tnp-core/browser';

var _a;
var Level;
(function (Level) {
    Level[Level["DATA"] = 0] = "DATA";
    Level[Level["INFO"] = 1] = "INFO";
    Level[Level["WARN"] = 2] = "WARN";
    Level[Level["ERROR"] = 3] = "ERROR";
    Level[Level["__NOTHING"] = 4] = "__NOTHING";
})(Level || (Level = {}));
var LevelKey = (_a = {},
    _a[Level.DATA] = 'log',
    _a[Level.INFO] = 'info',
    _a[Level.WARN] = 'warn',
    _a[Level.ERROR] = 'error',
    _a[Level.__NOTHING] = '',
    _a);
var LevelOrder = [
    LevelKey[Level.DATA],
    LevelKey[Level.INFO],
    LevelKey[Level.WARN],
    LevelKey[Level.ERROR],
];

var Display = /** @class */ (function () {
    function Display() {
    }
    Display.msg = function (message, params, moduleName, moduleColor, level, moduleWidth, isProductionMode) {
        if (isProductionMode) {
            return;
        }
        ;
        var color = 'gray';
        if (level === Level.INFO) {
            color = 'deepskyblue';
        }
        if (level === Level.ERROR) {
            color = 'red';
        }
        if (level === Level.WARN) {
            color = 'orange';
        }
        if (moduleWidth) {
            var diff = moduleWidth - moduleName.length;
            if (diff > 0) {
                for (var i = 0; i < diff; i++) {
                    moduleName += ' ';
                }
            }
        }
        if (Helpers.isBrowser) {
            var isEdgeOrIe8orAbove = (document['documentMode'] || /Edge/.test(navigator.userAgent));
            if (isEdgeOrIe8orAbove) {
                if (typeof message === 'string') {
                    var a1 = '[[ ' + moduleName + ' ]] ' + message + ' ';
                    params.unshift(a1);
                }
                else {
                    var a1 = '[[ ' + moduleName + ']] ';
                    params.push(message);
                    params.unshift(a1);
                }
                if (level === Level.INFO) {
                    console.info.apply(console, params);
                }
                else if (level === Level.ERROR) {
                    console.error.apply(console, params);
                }
                else if (level === Level.WARN) {
                    console.warn.apply(console, params);
                }
                else {
                    console.log.apply(console, params);
                }
            }
            else {
                if (typeof message === 'string') {
                    var a1 = '%c ' + moduleName + '  %c ' + message + ' ';
                    var a2 = 'background: ' + moduleColor + ';color:white; border: 1px solid ' + moduleColor + '; ';
                    var a3 = 'border: 1px solid ' + color + '; ';
                    params.unshift(a3);
                    params.unshift(a2);
                    params.unshift(a1);
                }
                else {
                    var a1 = '%c ' + moduleName + ' ';
                    var a2 = 'background: ' + moduleColor + ';color:white; border: 1px solid ' + color + '; ';
                    params.push(message);
                    params.unshift(a2);
                    params.unshift(a1);
                }
                console.log.apply(console, params);
            }
        }
        if (Helpers.isNode) {
        }
    };
    return Display;
}());

var Logger = /** @class */ (function () {
    function Logger(name, color, developmentMode, allowed, isMuted, fixedWidth) {
        var _this = this;
        this.name = name;
        this.color = color;
        this.developmentMode = developmentMode;
        this.allowed = allowed;
        this.isMuted = isMuted;
        this.fixedWidth = fixedWidth;
        /** @deprecated Use data(...)
         * @see data
        */
        this.d = function (name) {
            var data = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                data[_i - 1] = arguments[_i];
            }
            return _this._data(name, data);
        };
        /** @deprecated Use error(...)
         * @see error
        */
        this.er = function (name) {
            var data = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                data[_i - 1] = arguments[_i];
            }
            return _this._error(name, data);
        };
        /** @deprecated Use info(...)
         * @see info
        */
        this.i = function (name) {
            var data = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                data[_i - 1] = arguments[_i];
            }
            return _this._info(name, data);
        };
        /** @deprecated Use warn(...)
         * @see warn
        */
        this.w = function (name) {
            var data = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                data[_i - 1] = arguments[_i];
            }
            return _this._warn(name, data);
        };
        /**
         * Logs message and data with the level=data
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.data = function (message) {
            var otherParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                otherParams[_i - 1] = arguments[_i];
            }
            return _this._data(message, otherParams);
        };
        /**
         * Logs message and data with the level=error
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.error = function (message) {
            var otherParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                otherParams[_i - 1] = arguments[_i];
            }
            return _this._error(message, otherParams);
        };
        /**
         * Logs message and data with the level=info
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.info = function (message) {
            var otherParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                otherParams[_i - 1] = arguments[_i];
            }
            return _this._info(message, otherParams);
        };
        /**
         * Logs message and data with the level=warn
         * @param message The message
         * @param otherParams Additional parameters
         */
        this.warn = function (message) {
            var otherParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                otherParams[_i - 1] = arguments[_i];
            }
            return _this._warn(message, otherParams);
        };
    }
    Logger.prototype.setLevel = function (l) {
        this._level = l;
        return this;
    };
    Object.defineProperty(Logger.prototype, "isProductionMode", {
        get: function () {
            return !this.developmentMode;
        },
        enumerable: false,
        configurable: true
    });
    Logger.prototype.setProductionMode = function (productionMode) {
        this.developmentMode = !productionMode;
        return this;
    };
    Logger.prototype.mute = function () {
        this.isMuted = true;
        return this;
    };
    Logger.prototype.onlyWhen = function (expression) {
        if (typeof expression === 'function') {
            this.isMuted = !expression();
        }
        else if (typeof expression === 'boolean') {
            this.isMuted = !expression;
        }
    };
    Logger.prototype._data = function (name) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        if (this.isMuted)
            return this;
        if (this.allowed.length >= 1 && Helpers.contain(this.allowed, Level.__NOTHING)
            && !Helpers.contain(this.allowed, Level.DATA))
            return this;
        if (this.allowed.length === 0 || Helpers.contain(this.allowed, Level.DATA)) {
            Display.msg.apply(void 0, __spreadArray(__spreadArray([
                name
            ], __read(data)), [
                this.name,
                this.color,
                Level.DATA,
                this.fixedWidth,
                this.isProductionMode,
            ]));
        }
        return this;
    };
    Logger.prototype._error = function (name) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        if (this.isMuted)
            return this;
        if (this.allowed.length >= 1 && Helpers.contain(this.allowed, Level.__NOTHING)
            && !Helpers.contain(this.allowed, Level.ERROR))
            return this;
        if (this.allowed.length === 0 || Helpers.contain(this.allowed, Level.ERROR)) {
            Display.msg.apply(void 0, __spreadArray(__spreadArray([
                name
            ], __read(data)), [
                this.name,
                this.color,
                Level.ERROR,
                this.fixedWidth,
                this.isProductionMode,
            ]));
        }
        return this;
    };
    Logger.prototype._info = function (name) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        if (this.isMuted)
            return this;
        if (this.allowed.length >= 1 && Helpers.contain(this.allowed, Level.__NOTHING)
            && !Helpers.contain(this.allowed, Level.INFO))
            return this;
        if (this.allowed.length === 0 || Helpers.contain(this.allowed, Level.INFO)) {
            Display.msg.apply(void 0, __spreadArray(__spreadArray([
                name
            ], __read(data)), [
                this.name,
                this.color,
                Level.INFO,
                this.fixedWidth,
                this.isProductionMode,
            ]));
        }
        return this;
    };
    Logger.prototype._warn = function (name) {
        var data = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            data[_i - 1] = arguments[_i];
        }
        if (this.isMuted)
            return this;
        if (this.allowed.length >= 1 && Helpers.contain(this.allowed, Level.__NOTHING)
            && !Helpers.contain(this.allowed, Level.WARN))
            return this;
        if (this.allowed.length === 0 || Helpers.contain(this.allowed, Level.WARN)) {
            Display.msg.apply(void 0, __spreadArray(__spreadArray([
                name
            ], __read(data)), [
                this.name,
                this.color,
                Level.WARN,
                this.fixedWidth,
                this.isProductionMode,
            ]));
        }
        return this;
    };
    return Logger;
}());

var Log = /** @class */ (function () {
    //#region singleton
    function Log() {
        this._logOnly = false;
        this._logModules = false;
        this.isDevelopmentMode = true;
        this.modeIsSet = false;
        this.fixedWidth = 0;
        this.instances = {};
        this.levels = [];
        this.modules = [];
    }
    Object.defineProperty(Log, "instance", {
        // @ts-ignore
        get: function () {
            if (!Log['_instance']) {
                Log['_instance'] = new Log();
            }
            return Log['_instance'];
        },
        enumerable: false,
        configurable: true
    });
    Log.create = function (name) {
        var _a;
        var level = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            level[_i - 1] = arguments[_i];
        }
        return (_a = Log.instance).create.apply(_a, __spreadArray([name], __read(level)));
    };
    Log.disableLogs = function (level) {
        var _this = this;
        if (level === void 0) { level = Level.__NOTHING; }
        LevelOrder.reverse().find(function (a) {
            if (!_this.consolelogfn[a]) {
                _this.consolelogfn[a] = console[a];
            }
            console[a] = function () { };
            if (a === LevelKey[level]) {
                return true;
            }
            return false;
        });
    };
    Log.enableLogs = function () {
        var _this = this;
        LevelOrder.forEach(function (a) {
            console[a] = _this.consolelogfn[a];
        });
    };
    //#region public api
    Log.prototype.setProductionMode = function () {
        var _this = this;
        if (this.modeIsSet) {
            this.modeIsSet = false;
            throw '[ng2-logger] Production mode is already set';
        }
        else {
            this.modeIsSet = true;
            setTimeout(function () {
                if (_this.modeIsSet && console !== void 0 && console.clear !== void 0) {
                    console.clear();
                    console.log = function () { };
                    console.error = function () { };
                    console.warn = function () { };
                    console.info = function () { };
                }
            });
            this.isDevelopmentMode = false;
        }
    };
    Log.prototype.onlyModules = function () {
        var modules = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules[_i] = arguments[_i];
        }
        if (this._logModules) {
            throw '[ng2-logger] You should use funcion onlyModules only once';
        }
        if (!this._logModules) {
            this._logModules = true;
        }
        if (modules.length === 0)
            return;
        this.modules = modules;
        // console.log('only log this', Log.modules)
        this.muteAllOtherModules();
    };
    Log.prototype.onlyLevel = function () {
        var level = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            level[_i] = arguments[_i];
        }
        if (this._logOnly) {
            throw '[ng2-logger] You should use funcion onlyLevel only once';
        }
        if (!this._logOnly) {
            this._logOnly = true;
        }
        this.levels = Array.isArray(level) ? level : [level];
        for (var logName in this.instances) {
            if (this.instances.hasOwnProperty(logName)) {
                var element = this.instances[logName];
                element['allowed'] = this.levels;
            }
        }
    };
    //#endregion
    Log.prototype.create = function (name) {
        var level = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            level[_i - 1] = arguments[_i];
        }
        var i;
        if (Array.isArray(this.levels) && this.levels.length > 0) {
            level = this.levels;
        }
        if (this.instances[name] === void 0) {
            i = new (Log.Logger)(name, getRandomColor(), this.isDevelopmentMode, level, this.isMutedModule(name), this.levels.length > 0 ? this.fixedWidth : void 0);
            this.instances[name] = i;
        }
        else {
            i = this.instances[name];
        }
        return i;
    };
    Log.prototype.isMutedModule = function (moduleName) {
        if (this.modules.length == 0)
            return false;
        if (!Helpers.contain(this.modules, moduleName))
            return true;
        return false;
    };
    Log.prototype.muteAllOtherModules = function () {
        for (var moduleName in this.instances) {
            if (!Helpers.contain(this.modules, moduleName))
                this.instances[moduleName].mute();
        }
    };
    //#endregion
    Log.Logger = Logger;
    Log.consolelogfn = {};
    return Log;
}());
function getRandomColor() {
    var letters = '012345'.split('');
    var color = '#';
    color += letters[Math.round(Math.random() * 5)];
    letters = '0123456789ABCDEF'.split('');
    for (var i = 0; i < 5; i++) {
        color += letters[Math.round(Math.random() * 15)];
    }
    if (color === void 0) {
        return getRandomColor();
    }
    ;
    return color;
}

/**
 * Generated bundle index. Do not edit.
 */

export { Level, LevelKey, LevelOrder, Log, Logger };
//# sourceMappingURL=ng2-logger.js.map
